from typing import Any, List, Optional, Union


def filter_by_slice_or_index_or_indices(
    array: List, slice_or_index_or_indices: Optional[Union[slice, int, List[int]]] = None
):
    if isinstance(slice_or_index_or_indices, list):
        return list(map(lambda x: array[x], slice_or_index_or_indices))
    if isinstance(slice_or_index_or_indices, slice):
        return array[slice_or_index_or_indices]
    if isinstance(slice_or_index_or_indices, int):
        return [array[slice_or_index_or_indices]]
    return array


def convert_to_array_if_not(array_or_item: Union[List, Any]):
    return array_or_item if isinstance(array_or_item, list) else [array_or_item]



import numpy as np


class RoundNumericValuesMixin(object):
    # Default rounding precision in decimal places
    __round_precision__ = 9

    @classmethod
    def round_array_or_number(cls, array, decimal_places=__round_precision__):
        return np.round(array, cls.__round_precision__).tolist()



from mat3ra.utils import mixins as mixins

ARRAY_TO_ROUND = [1.23456789101112, 2.34567899, 3.45678999, 4.56789999, 5.67899999]


def test_RoundNumericValuesMixin():
    """
    Test the availability of the RoundNumericValuesMixin methods
    """

    class TestClass(mixins.RoundNumericValuesMixin):
        pass

    assert TestClass.round_array_or_number(ARRAY_TO_ROUND[0]) == 1.234567891
    assert TestClass.round_array_or_number(ARRAY_TO_ROUND) == [
        1.234567891,
        2.34567899,
        3.45678999,
        4.56789999,
        5.67899999,
    ]


def test_RoundNumericValuesMixin_inheritance():
    """
    Test the availability of the RoundNumericValuesMixin methods
    """

    class BaseClass(object):
        pass

    class TestClass(BaseClass, mixins.RoundNumericValuesMixin):
        __round_precision__ = 3

    assert TestClass.round_array_or_number(ARRAY_TO_ROUND[0]) == 1.235
    assert TestClass.round_array_or_number(ARRAY_TO_ROUND) == [
        1.235,
        2.346,
        3.457,
        4.568,
        5.679,
    ]
